/******************************************************************************
 *	File	:	platform.c
 *	Version :	1.1
 *	Date	:	2021/01/26
 *	Author	:	User
 *
 *	Hardware dependency function - C source file
 *
 *	Provide necessary function for EcmUsrDriver
 *
 * @copyright (C) 2020 NEXTW TECHNOLOGY CO., LTD.. All rights reserved.
 *
 ******************************************************************************/
#include <stdint.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <time.h>
#include <fcntl.h>
#include <pthread.h> 
#include "platform.h"
#include "EcmDriver.h"
#include "EcmUsrDriver.h"
#include "spidev.h"

#define ECM_USE_MUTEX	1

#define TRUE	1
#define FALSE	0

#define DEVNAME	"/dev/spidev2.0"

typedef unsigned char uchar;
typedef unsigned int uint;
typedef unsigned long ulong;
typedef uchar BOOL;

struct spi_ioc_transfer trans;

static int xf_handle = -1;
static uint32_t g_u32CrcType = ECM_CRC_TYPE_32;

#ifdef ECM_USE_MUTEX
pthread_mutex_t ecm_mutex;
#endif
/*************************************************
 * 
 * Local Bus module API 
 * 
 * **********************************************/

static int transfer(int fd, uint8_t const *tx, uint8_t const *rx, size_t len)
{
	int ret;
	int out_fd;
	struct spi_ioc_transfer tr = {
		.tx_buf = (unsigned long)tx,
		.rx_buf = (unsigned long)rx,
		.len = len,
		.speed_hz = trans.speed_hz,
		.bits_per_word = trans.bits_per_word,
	};

	ret = ioctl(fd, SPI_IOC_MESSAGE(1), &tr);
	if (ret < 1)
		printf("can't send spi message");
        return ret; 
}

/*************************************************
 * 
 * Polling DMA Done
 * 
 * **********************************************/

/*************************************************
 * 
 * ECM-XF Platform API 
 * 
 * **********************************************/
void UserSys_Init(uint32_t u32SpiFreq)
{
        int ret;
        uint32_t mode;
        struct sched_param param = {.sched_priority = 99};
	if(xf_handle < 0 && u32SpiFreq != 0){
		xf_handle = open( DEVNAME, O_RDONLY);
		if(xf_handle < 0){
                        printf("Init error: %d\n", xf_handle);
			return;
                }
                mode = SPI_MODE_0;
                trans.speed_hz = u32SpiFreq;
                trans.bits_per_word = 8;
	        ret = ioctl(xf_handle, SPI_IOC_WR_MODE32, &mode);
	        if (ret == -1)
		        printf("can't set spi mode");

	        /*
	         * bits per word
	         */
	        ret = ioctl(xf_handle, SPI_IOC_WR_BITS_PER_WORD, &trans.bits_per_word);
	        if (ret == -1)
		        printf("can't set bits per word");

	        /*
	         * max speed hz
	         */
	        ret = ioctl(xf_handle, SPI_IOC_WR_MAX_SPEED_HZ, &trans.speed_hz);
	        if (ret == -1)
		        printf("can't set max speed hz");
   
	}
        pthread_setschedparam( pthread_self(), SCHED_FIFO, &param);
}
void UserSys_Close(void)
{
	if(xf_handle >= 0)
		close(xf_handle);
	xf_handle = -1;
}
int UserSpiDataExchange(uint8_t *pTxBuf, uint8_t *pRxBuf, uint32_t u32PackSize)
{
        transfer(xf_handle, pTxBuf, pRxBuf, u32PackSize);
	return 1;
}
int UserDelay(uint32_t MircoSec)
{
	usleep(MircoSec);
	return 1;
}
// Optional function
int UserMutexInit(void)
{
#ifdef ECM_USE_MUTEX
	return pthread_mutex_init(&ecm_mutex, NULL);
#else
	return 0;
#endif
}
int UserMutexDestroy(void)
{
#ifdef ECM_USE_MUTEX
	return pthread_mutex_destroy(&ecm_mutex);
#else
	return 0;
#endif
}
int UserMutexLock(void)
{
#ifdef ECM_USE_MUTEX
	return pthread_mutex_lock(&ecm_mutex);
#else
	return 0;
#endif
}
int UserMutexTryLock(void)
{
#ifdef ECM_USE_MUTEX
	return pthread_mutex_trylock(&ecm_mutex);
#else
	return 0;
#endif
}
int UserMutexUnlock(void)
{
#ifdef ECM_USE_MUTEX
	return pthread_mutex_unlock(&ecm_mutex);
#else
	return 0;
#endif
}
uint32_t UserSetCrcType(uint32_t u32CrcType)
{
	g_u32CrcType = u32CrcType;
	return g_u32CrcType;
}
uint32_t UserCalCrc(uint8_t *pu8Addr, uint32_t u32Size)
{
	return ECM_CRC_MAGIC_NUM;
}
//	(C) COPYRIGHT 2020 NEXTW Technology Corp.
